# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaImages - an image factory.
## This class is not ment to be instanciated. Its only purpose is to
## provide named tcl image objects via YadaImages::get.
##
## These are the symbolic image names known to YadaImages:
## \latexonly
## back, bomb, checkmark, folder, delete, down, erase, error, eyes, finish,
## floppy, forward, giveup, halfeyes, home, info, lamp, ledOff, ledOn, yadalogo,
## open, folder\_open, question, reload, run, runall, step, stop, text
## \endlatexonly
## ----------------------------------------------------------------------------
## \image html back.gif "backward"
## \image html bomb.gif "bomb"
## \image html checkmark.gif "checkmark"
## \image html folder.gif "defaultFolder"
## \image html delete.gif "delete"
## \image html down.gif "down"
## \image html erase.gif "erase"
## \image html error.gif "error"
## \image html eyes.gif "eyes"
## \image html finish.gif "finish"
## \image html floppy.gif "floppy"
## \image html forward.gif "forward"
## \image html giveup.gif "giveup"
## \image html halfeyes.gif "halfeyes"
## \image html home.gif "home"
## \image html info.gif "info"
## \image html lamp.gif "lamp"
## \image html ledOff.gif "ledOff"
## \image html ledOn.gif "ledOn"
## \image html yadalogo.gif "logo"
## \image html open.gif "openFile"
## \image html folder_open.gif "openFolder"
## \image html question.gif "question"
## \image html reload.gif "reload"
## \image html run.gif "run"
## \image html runall.gif "runall"
## \image html step.gif "step"
## \image html stop.gif "stop"
## \image html text.gif "textFile"
## \image html up.gif "up"
## ----------------------------------------------------------------------------
##
## \author Michael Daum
##
## $Id: YadaImages.tcl,v 1.11 2004/02/25 14:42:08 micha Exp $
## ----------------------------------------------------------------------------
class YadaImages {

  # variables ----------------------------------------------------------------

  ## hash mapping image names to tcl image objects
  common _images; ## \type TclArray

  array set _images [list \
    home		[image create photo -file [file join $env(YADA_IMAGES) home.gif]] \
    defaultFolder	[image create photo -file [file join $env(YADA_IMAGES) folder.gif]] \
    openFolder		[image create photo -file [file join $env(YADA_IMAGES) folder_open.gif]] \
    textFile		[image create photo -file [file join $env(YADA_IMAGES) text.gif]] \
    backward		[image create photo -file [file join $env(YADA_IMAGES) back.gif]] \
    forward		[image create photo -file [file join $env(YADA_IMAGES) forward.gif]] \
    lamp		[image create photo -file [file join $env(YADA_IMAGES) lamp.gif]] \
    up			[image create photo -file [file join $env(YADA_IMAGES) up.gif]] \
    down		[image create photo -file [file join $env(YADA_IMAGES) down.gif]] \
    floppy		[image create photo -file [file join $env(YADA_IMAGES) floppy.gif]] \
    openFile		[image create photo -file [file join $env(YADA_IMAGES) open.gif]] \
    reload		[image create photo -file [file join $env(YADA_IMAGES) reload.gif]] \
    erase		[image create photo -file [file join $env(YADA_IMAGES) erase.gif]] \
    eyes		[image create photo -file [file join $env(YADA_IMAGES) eyes.gif]] \
    halfeyes		[image create photo -file [file join $env(YADA_IMAGES) halfeyes.gif]] \
    giveup		[image create photo -file [file join $env(YADA_IMAGES) giveup.gif]] \
    run			[image create photo -file [file join $env(YADA_IMAGES) run.gif]] \
    runall		[image create photo -file [file join $env(YADA_IMAGES) runall.gif]] \
    step		[image create photo -file [file join $env(YADA_IMAGES) step.gif]] \
    finish		[image create photo -file [file join $env(YADA_IMAGES) finish.gif]] \
    stop		[image create photo -file [file join $env(YADA_IMAGES) stop.gif]] \
    bomb		[image create photo -file [file join $env(YADA_IMAGES) bomb.gif]] \
    delete		[image create photo -file [file join $env(YADA_IMAGES) delete.gif]] \
    ledOn		[image create photo -file [file join $env(YADA_IMAGES) ledOn.gif]] \
    ledOff		[image create photo -file [file join $env(YADA_IMAGES) ledOff.gif]] \
    checkmark		[image create photo -file [file join $env(YADA_IMAGES) checkmark.gif]] \
    question		[image create photo -file [file join $env(YADA_IMAGES) question.gif]] \
    info		[image create photo -file [file join $env(YADA_IMAGES) info.gif]] \
    error		[image create photo -file [file join $env(YADA_IMAGES) error.gif]] \
    logo		[image create photo -file [file join $env(YADA_IMAGES) yadalogo.gif]] \
  ]

  # static methods ------------------------------------------------------------
  public proc get {imageName}; ## \type TclString
};

## ----------------------------------------------------------------------------
## get named image.
## This method lets you access the stored image objects by their name.
## ----------------------------------------------------------------------------
proc YadaImages::get {imageName} {
  return $_images($imageName)
}
