# Copyright (C) 1997-2004 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaMachines - configure document for all machines.
## \ingroup YadaConfiguration
##
## \author Michael Daum
##
## $Id: YadaMachines.tcl,v 1.12 2004/10/15 17:24:37 micha Exp $
## ----------------------------------------------------------------------------
class YadaMachines {
  inherit YadaConfigDocument

  # methods ------------------------------------------------------------------
  public method init {}
  public method load {}
  public method save {}

  constructor {args} {}; ## \type TclList

  private method _commit {}
  private method _defaults {{fileName ""}}; ## \type TclString
  private method _new {}
  private method _delete {}
  private method _select {{machineName ""}}; ## \type TclString

};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaMachines::constructor {args} {

  set _allAspects {name nodes access cpus}



  # nodes 
  itk_component add nodesLabel {
    label $itk_component(childsite).nodesLabel \
      -text "Nodes:" \
      -anchor nw
  } {}

  itk_component add nodesMark {
    frame $itk_component(childsite).nodesMark \
      -borderwidth 2
  } {}

  itk_component add nodes {
    iwidgets::combobox $itk_component(nodesMark).box \
      -completion false \
      -unique true \
      -borderwidth 2 \
  }  {}
  set entry [$itk_component(nodes) component entry]
  $entry configure \
    -validate all \
    -validatecommand [code $this _validateCommand "nodes" %P]

  # access
  itk_component add accessLabel {
    label $itk_component(childsite).accessLabel \
      -text "Access:" \
      -anchor nw
  } {}

  itk_component add accessMark {
    frame $itk_component(childsite).accessMark \
      -borderwidth 2
  } {}

  itk_component add access {
    iwidgets::combobox $itk_component(accessMark).box \
      -completion false \
      -unique true \
      -borderwidth 2 \
  }  {}
  set entry [$itk_component(access) component entry]
  $entry configure \
    -validate all \
    -validatecommand [code $this _validateCommand "access" %P]

  # cpus 
  itk_component add cpusLabel {
    label $itk_component(childsite).cpusLabel \
      -text "CPUs:" \
      -anchor nw 
  } {}

  itk_component add cpusMark {
    frame $itk_component(childsite).cpusMark \
      -borderwidth 2
  } {}

  itk_component add cpus {
    entry $itk_component(cpusMark).cpus \
      -borderwidth 2 \
      -validatecommand [code $this _validateCommand "cpus" %P] \
      -validate all
  }  {}

  # load 
  itk_component add loadLabel {
    label $itk_component(childsite).loadLabel \
      -text "Current Load:" \
      -anchor nw 
  } {}

  itk_component add loadMark {
    frame $itk_component(childsite).loadMark \
      -borderwidth 2
  } {}

  itk_component add load {
    entry $itk_component(loadMark).load \
      -borderwidth 2 \
      -state disabled 
  } {}

  #
  # packing + gridding
  #
  grid rowconfigure $itk_component(childsite) 4 -weight 1
  grid configure $itk_component(nameMark) -columnspan 3
  
  grid $itk_component(nodesLabel) -sticky new -row 1 -column 0 -pady 10
  grid $itk_component(nodesMark) -sticky new -row 1 -column 1 -pady 10 -columnspan 3
  pack $itk_component(nodes) -fill both -expand 1 -side left

  grid $itk_component(accessLabel) -sticky new -row 2 -column 0 -pady 10
  grid $itk_component(accessMark) -sticky new -row 2 -column 1 -pady 10 -columnspan 3
  pack $itk_component(access) -fill both -expand 1 -side left

  grid $itk_component(cpusLabel) -sticky ew -row 3 -column 0 -pady 10
  grid $itk_component(cpusMark) -sticky w -row 3 -column 1 -pady 10
  pack $itk_component(cpus) -fill both -expand 1 -side left

  grid $itk_component(loadLabel) -sticky ew -row 3 -column 2 -pady 10
  grid $itk_component(loadMark) -sticky w -row 3 -column 3 -pady 10
  pack $itk_component(load) -fill both -expand 1 -side left

  eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## init
## ----------------------------------------------------------------------------
body YadaMachines::init {} {
  if {$_isInitialized} {
    return
  }
  chain

  # initialize the nodes combobox 
  set nodesList ""
  set accessList ""
  foreach machine [getAllItems] {
    set nodes [$machine cget -nodes]
    set access [$machine cget -access]
    if {[lsearch $nodesList $nodes] < 0} {
      lappend nodesList $nodes
    }
    if {[lsearch $accessList $access] < 0} {
      lappend accessList $access
    }
  }
  eval $itk_component(nodes) insert list end $nodesList
  eval $itk_component(access) insert list end $accessList


  #set machines [itcl_info objects -class YadaMachine]
  #foreach machine $machines {
  #    if { [$machine hasSavedState] } {
  #	  $machine load
  #    } 
  #}

  _select "<none>"
}

## ----------------------------------------------------------------------------
## _select
## ----------------------------------------------------------------------------
body YadaMachines::_select {{machineName ""}} {

  # get a machine name from the combobox if we didn't provide one
  if {$machineName == ""} {
    set machineName [$itk_component(name) get]
  } 

  set currentItem [getItem $machineName]
  if {$currentItem == ""} {
    printMessage "ERROR: machine `$machineName' is unknown" error
    return
  }

  # check if we didn't commit changes
  if {![_askChange]} {
    return
  }


  # set the active machine
  setCurrentItem $currentItem

  ## update title bar 
  displayTitle

  # update the name box
  foreach aspect {name nodes access} {
    set box $itk_component($aspect)
    set values [[$box component list] get 0 end]
    set value [$currentItem cget -$aspect]
    set index [lsearch $values $value]
    if {$index > 0} {
      $box selection set $index $index
    }
    $box delete entry 0 end
    $box insert entry 0 $value
  }

  # update the entries
  foreach aspect {cpus} {
    set entry $itk_component($aspect) 
    $entry delete 0 end
    $entry insert 0 [$currentItem cget -$aspect]
  }
  set entry $itk_component(load) 
  $entry configure -state normal
  $entry delete 0 end
  $entry insert 0 [$currentItem getLoad]
  $entry configure -state disabled

  # remove marking
  _unmark


  # remember this selected name
  set _oldItem $machineName

}

## ----------------------------------------------------------------------------
## _defaults
## ----------------------------------------------------------------------------
body YadaMachines::_defaults {{fileName ""}} {
  global env

  # default
  if {$fileName == ""} {
    set fileName [file join $env(YADA_LIB) machines.tcl.sample]
  } 

  # ... configured machines
  if {[catch {source $fileName} errMsg]} {
    bgerror "There's an error in your machines init file $fileName!"
  }

  # required localhost machine
  YadaMachine ::#auto -name "localhost" -nodes "127.0.0.1"
  YadaMachine ::#auto -name "<none>" -nodes "" -access "" -cpus 0
  
  # assert machines into the class
  set machines [itcl_info objects -class YadaMachine]
  foreach machine $machines {
    set machineName [$machine cget -name]
    set item [getItem $machineName]
    if {$item != ""} {
      puts "WARNING: machine `$machineName' already exists"
    }

    setItem $machine
  }

  .main printStatus "... built [llength $machines] machines"
}

## ----------------------------------------------------------------------------
## _commit
## ----------------------------------------------------------------------------
body YadaMachines::_commit {} {

  set currentItem [getCurrentItem]
  set oldName [$currentItem cget -name]

  if {![chain]} {
    return
  }

  # adjust each experiment using a machine with a changed name
  set newName [$currentItem cget -name]
  if {$oldName != $newName} {
    foreach experiment [.main getExperiments] {
      if {[$experiment cget -machineName] == $oldName} {
	$experiment configure -machineName $newName
      }
    }
  }

  # save all aspects
  foreach aspect {nodes access cpus} {
    $currentItem configure -$aspect [$itk_component($aspect) get]
  }

  # save data unique in comboboxes
  foreach aspect {nodes access} {
    set box $itk_component($aspect)
    set value [$currentItem cget -$aspect]
    set listBox [$box component list]
    if {[getIndexOfName $listBox $value] < 0} {
      set entries [$listBox get 0 end] 
      lappend entries $value
      $listBox delete 0 end
      eval $listBox insert 0 [lsort -dictionary $entries]
    }
  }

  # remember this selected name
  set _oldItem [$currentItem cget -name]

  ## Mark as modified
  $currentItem setModified

  # remove marking
  _unmark
  _select
}

## ----------------------------------------------------------------------------
## _delete.
## \todo not yet implemented
## ----------------------------------------------------------------------------
body YadaMachines::_delete {} {
}

## ----------------------------------------------------------------------------
## _new.
## \todo not yet implemented
## ----------------------------------------------------------------------------
body YadaMachines::_new {} {
}


## ----------------------------------------------------------------------------
## Overriding default implementation. Delegates command to current machine's
## load method
## ----------------------------------------------------------------------------
body YadaMachines::load {} {
    set currentItem [getCurrentItem]

    if { [$currentItem cget -name] != "<none>"} {
	[getCurrentItem] load
	_select
	displayTitle
    }
}

## ----------------------------------------------------------------------------
## Overriding default implementation. Delegates command to current machine's
## save method
## ----------------------------------------------------------------------------
body YadaMachines::save {} {
    set currentItem [getCurrentItem]

    if { [$currentItem cget -name] != "<none>"} {
	[getCurrentItem] save
	displayTitle
    }
}





